// +build ignore

/*
	Note: This file is autogenerated! Do not edit it manually!
	Edit client_resource_template.go instead, and run
	hack/generate-client.sh afterwards.
*/

package client

import (
	log "github.com/sirupsen/logrus"
	api "github.com/weaveworks/ignite/pkg/apis/ignite/v1alpha1"
	meta "github.com/weaveworks/ignite/pkg/apis/meta/v1alpha1"
	"github.com/weaveworks/ignite/pkg/storage"
	"github.com/weaveworks/ignite/pkg/storage/filterer"
)

// ResourceClient is an interface for accessing Resource-specific API objects
type ResourceClient interface {
	// Get returns the Resource matching given UID from the storage
	Get(meta.UID) (*api.Resource, error)
	// Set saves the given Resource into persistent storage
	Set(*api.Resource) error
	// Find returns the Resource matching the given filter, filters can
	// match e.g. the Object's Name, UID or a specific property
	Find(filter filterer.BaseFilter) (*api.Resource, error)
	// FindAll returns multiple Resources matching the given filter, filters can
	// match e.g. the Object's Name, UID or a specific property
	FindAll(filter filterer.BaseFilter) ([]*api.Resource, error)
	// Delete deletes the Resource with the given UID from the storage
	Delete(uid meta.UID) error
	// List returns a list of all Resources available
	List() ([]*api.Resource, error)
}

// Resources returns the ResourceClient for the Client instance
func (c *Client) Resources() ResourceClient {
	if c.resourceClient == nil {
		c.resourceClient = newResourceClient(c.storage)
	}

	return c.resourceClient
}

// Resources is a shorthand for accessing Resources using the default client
func Resources() ResourceClient {
	return DefaultClient.Resources()
}

// resourceClient is a struct implementing the ResourceClient interface
// It uses a shared storage instance passed from the Client together with its own Filterer
type resourceClient struct {
	storage  storage.Storage
	filterer *filterer.Filterer
}

// newResourceClient builds the resourceClient struct using the storage implementation and a new Filterer
func newResourceClient(s storage.Storage) ResourceClient {
	return &resourceClient{
		storage:  s,
		filterer: filterer.NewFilterer(s),
	}
}

// Find returns a single Resource based on the given Filter
func (c *resourceClient) Find(filter filterer.BaseFilter) (*api.Resource, error) {
	object, err := c.filterer.Find(api.ResourceKind, filter)
	if err != nil {
		return nil, err
	}

	return object.(*api.Resource), nil
}

// FindAll returns multiple Resources based on the given Filter
func (c *resourceClient) FindAll(filter filterer.BaseFilter) ([]*api.Resource, error) {
	matches, err := c.filterer.FindAll(api.ResourceKind, filter)
	if err != nil {
		return nil, err
	}

	results := make([]*api.Resource, 0, len(matches))
	for _, item := range matches {
		results = append(results, item.(*api.Resource))
	}

	return results, nil
}

// Get returns the Resource matching given UID from the storage
func (c *resourceClient) Get(uid meta.UID) (*api.Resource, error) {
	log.Debugf("Client.Get; UID: %q, Kind: %s", uid, api.KindResource)
	object, err := c.storage.GetByID(api.KindResource, uid)
	if err != nil {
		return nil, err
	}

	return object.(*api.Resource), nil
}

// Set saves the given Resource into the persistent storage
func (c *resourceClient) Set(resource *api.Resource) error {
	log.Debugf("Client.Set; UID: %q, Kind: %s", resource.GetUID(), resource.GetKind())
	return c.storage.Set(resource)
}

// Delete deletes the Resource from the storage
func (c *resourceClient) Delete(uid meta.UID) error {
	log.Debugf("Client.Delete; UID: %q, Kind: %s", uid, api.KindResource)
	return c.storage.Delete(api.KindResource, uid)
}

// List returns a list of all Resources available
func (c *resourceClient) List() ([]*api.Resource, error) {
	list, err := c.storage.List(api.KindResource)
	if err != nil {
		return nil, err
	}

	results := make([]*api.Resource, 0, len(list))
	for _, item := range list {
		results = append(results, item.(*api.Resource))
	}

	return results, nil
}
